package com.qmth.boot.core.metrics.config;

import com.qmth.boot.core.constant.CoreConstant;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;

import javax.validation.constraints.NotEmpty;
import java.time.Duration;

/**
 * 统计工具参数设置
 */
@Component
@ConfigurationProperties(prefix = CoreConstant.CONFIG_PREFIX + ".metrics")
@Validated
public class MetricsProperties {

    /**
     * 默认时间分组，按0~10ms，10ms~100ms，100ms~500ms，500ms~1s，1s~10s，>10s分为6个档次
     */
    @NotEmpty
    private Duration[] timeGroup = new Duration[] { Duration.ofMillis(10), Duration.ofMillis(100),
            Duration.ofMillis(500), Duration.ofSeconds(1), Duration.ofSeconds(10) };

    /**
     * 获取响应时间分组间隔
     *
     * @return
     */
    public Duration[] getTimeGroup() {
        return timeGroup;
    }

    public void setTimeGroup(Duration[] timeGroup) {
        this.timeGroup = timeGroup;
    }
}
