package com.qmth.boot.core.metrics.service.impl;

import com.qmth.boot.core.metrics.config.MetricsProperties;
import com.qmth.boot.core.metrics.model.MetricsResult;
import com.qmth.boot.core.metrics.service.MetricsService;

import javax.annotation.PreDestroy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MemoryMetricsService implements MetricsService {

    private Map<String, MetricsRecorder> recorderMap;

    private MetricsProperties metricsProperties;

    public MemoryMetricsService(MetricsProperties metricsProperties) {
        this.metricsProperties = metricsProperties;
        this.recorderMap = new HashMap<>();
    }

    @PreDestroy
    public void close() {
        this.recorderMap.clear();
    }

    @Override
    public void record(String endpoint, int status, long timecost) {
        recorderMap.computeIfAbsent(endpoint, key -> new MetricsRecorder(metricsProperties)).record(status, timecost);
    }

    @Override
    public MetricsResult result(String endpoint) {
        MetricsResult result = new MetricsResult();
        MetricsRecorder recorder = recorderMap.get(endpoint);
        if (recorder != null) {
            result = recorder.getResult();
        }
        result.setEndpoint(endpoint);
        return result;
    }

    @Override
    public List<MetricsResult> result() {
        List<MetricsResult> list = new ArrayList<>();
        for (Map.Entry<String, MetricsRecorder> entry : recorderMap.entrySet()) {
            MetricsResult result = entry.getValue().getResult();
            result.setEndpoint(entry.getKey());
            list.add(result);
        }
        return list;
    }

    @Override
    public void reset(String endpoint) {
        recorderMap.remove(endpoint);
    }

    @Override
    public void reset() {
        recorderMap.clear();
    }
}
